'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _plugins = require('./plugins');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**************************************************
 * Created by nanyuantingfeng on 27/10/2017 16:28.
 **************************************************/
function fnFixStyleLoaders4Production(rules, isProduction) {
  var styleLoader = 'style-loader';
  if (isProduction) {
    return _plugins.ExtractTextPlugin.extract({ fallback: styleLoader, use: rules });
  }
  return [{ loader: styleLoader }].concat((0, _toConsumableArray3.default)(rules));
}

function fnGetTheme(packageMap, cwd) {
  var theme = {};
  var packageMapTheme = packageMap.theme;

  if (packageMapTheme && typeof packageMapTheme === 'string') {

    var pp = packageMapTheme;

    if (pp.charAt(0) === '.') {
      pp = _path2.default.resolve(cwd, pp);
    }

    var fn = require(pp);
    theme = fn();
  } else if (packageMapTheme && (typeof packageMapTheme === 'undefined' ? 'undefined' : (0, _typeof3.default)(packageMapTheme)) === 'object') {
    theme = packageMapTheme;
  }

  return theme;
}

exports.default = function () {
  var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(context, next) {
    var cwd, _context$limit, limit, ENV, packageMap, isProduction, theme, babelOptions, postcssOptions, tsOptions, rules;

    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            context.rules = [];
            next();

            cwd = context.cwd, _context$limit = context.limit, limit = _context$limit === void 0 ? 10000 : _context$limit, ENV = context.ENV, packageMap = context.packageMap;
            isProduction = ENV.isProduction;
            theme = fnGetTheme(packageMap, cwd);
            babelOptions = context.babelOptions, postcssOptions = context.postcssOptions, tsOptions = context.tsOptions, rules = context.rules;


            context.rules = [{
              test: function test(filePath) {
                return (/\.lazy\.jsx?$/.test(filePath)
                );
              },

              exclude: /node_modules/,
              use: [{ loader: 'babel-loader', options: babelOptions }, { loader: 'bundle-loader', options: { lazy: true } }]
            }, {
              test: /\.js$/,
              exclude: /node_modules/,
              use: [{ loader: 'babel-loader', options: babelOptions }]
            }, {
              test: /\.jsx$/,
              use: [{ loader: 'babel-loader', options: babelOptions }]
            }, {
              test: /\.tsx?$/,
              exclude: /node_modules/,
              use: [{ loader: 'babel-loader', options: babelOptions }, { loader: 'ts-loader', options: tsOptions }]
            }, {
              test: function test(filePath) {
                return (/\.css$/.test(filePath) && !/\.module\.css$/.test(filePath)
                );
              },

              use: fnFixStyleLoaders4Production([{
                loader: 'css-loader', options: {
                  sourceMap: true,
                  '-autoprefixer': true,
                  '-restructuring': true
                }
              }, { loader: 'postcss-loader', options: postcssOptions }], isProduction)
            }, {
              test: /\.module\.css$/,
              use: fnFixStyleLoaders4Production([{
                loader: 'css-loader', options: {
                  sourceMap: true,
                  modules: true,
                  localIdentName: '[local]___[hash:base64:5]',
                  '-autoprefixer': true,
                  '-restructuring': true
                }
              }, { loader: 'postcss-loader', options: postcssOptions }], isProduction)
            }, {
              test: function test(filePath) {
                return (/\.less$/.test(filePath) && !/\.module\.less$/.test(filePath)
                );
              },

              use: fnFixStyleLoaders4Production([{
                loader: 'css-loader', options: {
                  sourceMap: true,
                  '-autoprefixer': true
                }
              }, { loader: 'postcss-loader', options: postcssOptions }, { loader: 'less-loader', options: { sourceMap: true, modifyVars: theme } }], isProduction)
            }, {
              test: /\.module\.less$/,
              use: fnFixStyleLoaders4Production([{
                loader: 'css-loader', options: {
                  sourceMap: true,
                  modules: true,
                  localIdentName: '[local]___[hash:base64:5]',
                  '-autoprefixer': true
                }
              }, { loader: 'postcss-loader', options: postcssOptions }, { loader: 'less-loader', options: { sourceMap: true, modifyVars: theme } }], isProduction)
            }, {
              test: /\.woff(\?v=\d+\.\d+\.\d+)?$/,
              use: [{ loader: 'url-loader', options: { limit: limit, minetype: 'application/font-woff' } }]
            }, {
              test: /\.woff2(\?v=\d+\.\d+\.\d+)?$/,
              use: [{ loader: 'url-loader', options: { limit: limit, minetype: 'application/font-woff' } }]
            }, {
              test: /\.ttf(\?v=\d+\.\d+\.\d+)?$/,
              use: [{ loader: 'url-loader', options: { limit: limit, minetype: 'application/octet-stream' } }]
            }, {
              test: /\.eot(\?v=\d+\.\d+\.\d+)?$/,
              use: [{ loader: 'url-loader', options: { limit: limit, minetype: 'application/vnd.ms-fontobject' } }]
            }, {
              test: /\.svg(\?v=\d+\.\d+\.\d+)?$/,
              use: [{ loader: 'url-loader', options: { limit: limit, minetype: 'image/svg+xml' } }]
            }, {
              test: /\.(png|jpg|jpeg|gif)(\?v=\d+\.\d+\.\d+)?$/i,
              use: [{ loader: 'url-loader', options: { limit: limit } }]
            }, {
              test: /\.html?$/,
              use: [{
                loader: 'file-loader', options: {
                  name: '[path][name].[ext]'
                }
              }]
            }, {
              test: /\.hbs?$/, use: [{ loader: 'mustache-loader' }]
            }].concat((0, _toConsumableArray3.default)(rules));

          case 7:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  return function (_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

module.exports = exports['default'];